using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace System
{
    public class HTML
    {
        public static bool Abort = false;

        public static WebBrowser Navigate(WebBrowser webBrowser, string url, int secondsTimeout)
        {
            return WebNavigator.Navigate(webBrowser, url, secondsTimeout);
        }

        public static WebBrowser Navigate(string url, int secondsTimeout)
        {
            return WebNavigator.Navigate(url, secondsTimeout);
        }

        public static WebBrowser WaitDocumentCompleted(WebBrowser webBrowser, int secondsTimeout)
        {
            return WebNavigator.WaitDocumentCompleted(webBrowser, secondsTimeout);
        }

        #region class WebNavigator for method Navigate

        private class WebNavigator: IDisposable
        {
            public static WebBrowser WaitDocumentCompleted(WebNavigator navigator, int secondsTimeout)
            {
                if(navigator != null) return navigator.WaitDocumentCompleted(secondsTimeout);
                return null;
            }

            public static WebBrowser WaitDocumentCompleted(WebBrowser webBrowser, int secondsTimeout)
            {
                WebNavigator navigator = new WebNavigator(webBrowser, "");
                WebBrowser result = navigator.WaitDocumentCompleted(secondsTimeout);
                navigator.Dispose();
                return result;
            }

            public static WebBrowser Navigate(WebBrowser webBrowser, string url, int secondsTimeout)
            {
                WebNavigator navigator = new WebNavigator(webBrowser, url);
                WebBrowser result = navigator.WaitDocumentCompleted(secondsTimeout);
                navigator.Dispose();
                return result;
            }

            public static WebBrowser Navigate(string url, int secondsTimeout)
            {
                return Navigate(null, url, secondsTimeout);
            }

            public WebBrowser webBrowser = null;
            public bool documentCompleted = false;

            public WebNavigator(WebBrowser webBrowser, string url)
            {
                this.webBrowser = (webBrowser != null ? webBrowser : new ExtendedWebBrowser());
                this.webBrowser.ScriptErrorsSuppressed = true;
                documentCompleted = false;
                if(this.webBrowser != null) {
                    this.webBrowser.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(webBrowser_DocumentCompleted);
                    if(url != null && url.Length > 0) this.webBrowser.Navigate(url);
                }
            }
            public WebNavigator(WebBrowser webBrowser): this(webBrowser, "")
            {
            }

            private void webBrowser_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
            {
                documentCompleted = true;
                this.webBrowser.DocumentCompleted -= webBrowser_DocumentCompleted;
            }

            public WebBrowser WaitDocumentCompleted(int secondsTimeout)
            {
                int startTickCount = Environment.TickCount;
                while((Environment.TickCount - startTickCount) < secondsTimeout * 1000 && documentCompleted == false) {
                    if(HTML.Abort) return null;
                    Application.DoEvents();
                }
                return (documentCompleted && webBrowser != null && webBrowser.Document != null ? webBrowser : null);
            }

            public void Dispose()
            {
                if(this.webBrowser != null) this.webBrowser.DocumentCompleted -= webBrowser_DocumentCompleted;
            }
        }

        #endregion

        public static HtmlElement GetElementById(HtmlDocument document, string elementId)
        {
            if(document != null) {
                try {
                    HtmlElement el = document.GetElementById(elementId);
                    if(el != null) return el;
                    if(document.Window.Frames.Count > 0)
                        foreach(HtmlWindow window in document.Window.Frames) {
                            el = GetElementById(window.Document, elementId);
                            if(el != null) return el;
                        }
                } catch { }
            }
            return null;
        }
        public static HtmlElement GetElementById(WebBrowser webBrowser, string elementId)
        {
            return GetElementById(webBrowser != null ? webBrowser.Document : null, elementId);
        }

        public static HtmlElement GetPrevElement(HtmlElement el)
        {
            try {
                HtmlDocument doc = el.Document;
                HtmlElement prev = null;
                foreach(HtmlElement e in doc.All) {
                    if(e.Equals(el)) return prev;
                    prev = e;
                }
            } catch { }
            return null;
        }

        public static HtmlElement GetNextElement(HtmlElement el)
        {
            try {
                HtmlDocument doc = el.Document;
                Boolean returnNext = false;
                foreach(HtmlElement e in doc.All) {
                    if(returnNext) return e;
                    if(e.Equals(el)) returnNext = true;
                }
            } catch { }
            return null;
        }

        public static HtmlElement GetElementByAttributeValue(HtmlDocument document, string tag, string attribute, string value)
        {
            if(document != null) {
                try {
                    if(attribute.Length == 0 || value.Length == 0) return null;
                    if(tag.Length > 0) {
                        foreach(HtmlElement el in document.All)
                            if(el.GetAttribute(attribute) == value) return el;
                    } else {
                        foreach(HtmlElement el in document.All)
                            if(el.TagName == tag && el.GetAttribute(attribute) == value) return el;
                    }
                    if(document.Window.Frames.Count > 0)
                        foreach(HtmlWindow window in document.Window.Frames) {
                            HtmlElement el = GetElementByAttributeValue(window.Document, tag, attribute, value);
                            if(el != null) return el;
                        }
                } catch { }
            }
            return null;
        }
        public static HtmlElement GetElementByAttributeValue(WebBrowser webBrowser, string tag, string attribute, string value)
        {
            return GetElementByAttributeValue(webBrowser != null ? webBrowser.Document : null, tag, attribute, value);
        }

        public static HtmlElement GetElementByAttributeValue(HtmlDocument document, string tag, string attribute, string startsWith, string contains)
        {
            if(document != null) {
                try {
                    if(startsWith.Length > 0 && contains.Length > 0) {
                        if(tag.Length == 0) {
                            foreach(HtmlElement el in document.All) {
                                String attributeValue = el.GetAttribute(attribute);
                                if(attributeValue.StartsWith(startsWith) && attributeValue.Contains(contains)) return el;
                            }
                        } else {
                            foreach(HtmlElement el in document.All) {
                                String attributeValue = el.GetAttribute(attribute);
                                if(el.TagName == tag && attributeValue.StartsWith(startsWith) && attributeValue.Contains(contains)) return el;
                            }
                        }
                    } else if(startsWith.Length > 0) {
                        if(tag.Length == 0) {
                            foreach(HtmlElement el in document.All)
                                if(el.GetAttribute(attribute).StartsWith(startsWith)) return el;
                        } else {
                            foreach(HtmlElement el in document.All)
                                if(el.TagName == tag && el.GetAttribute(attribute).StartsWith(startsWith)) return el;
                        }
                    } else if(contains.Length > 0) {
                        if(tag.Length == 0) {
                            foreach(HtmlElement el in document.All)
                                if(el.GetAttribute(attribute).Contains(contains)) return el;
                        } else {
                            foreach(HtmlElement el in document.All)
                                if(el.TagName == tag && el.GetAttribute(attribute).Contains(contains)) return el;
                        }
                    } else {
                        return null;
                    }
                    if(document.Window.Frames.Count > 0)
                        foreach(HtmlWindow window in document.Window.Frames) {
                            HtmlElement el = GetElementByAttributeValue(window.Document, tag, startsWith, contains);
                            if(el != null) return el;
                        }
                } catch { }
            }
            return null;
        }
        public static HtmlElement GetElementByAttributeValue(WebBrowser webBrowser, string tag, string attribute, string startsWith, string contains)
        {
            return GetElementByAttributeValue(webBrowser != null ? webBrowser.Document : null, tag, attribute, startsWith, contains);
        }

        public static HtmlElement GetElementThatContains(HtmlDocument document, string tag, string contains)
        {
            if(document == null) return null;
            try {
                if(contains.Length == 0) return null;
                if(tag.Length == 0) {
                    foreach(HtmlElement el in document.All)
                        if(el.OuterHtml.Contains(contains)) return el;
                } else {
                    foreach(HtmlElement el in document.All)
                        if(el.TagName == tag && el.OuterHtml.Contains(contains)) return el;
                }
                if(document.Window.Frames.Count > 0)
                    foreach(HtmlWindow window in document.Window.Frames) {
                        HtmlElement el = GetElementThatContains(window.Document, tag, contains);
                        if(el != null) return el;
                    }
            } catch { }
            return null;
        }
        public static HtmlElement GetElementThatContains(WebBrowser webBrowser, string tag, string contains)
        {
            return GetElementThatContains(webBrowser != null ? webBrowser.Document : null, tag, contains);
        }

        public static HtmlElement GetElementThatContains(HtmlDocument document, string tag, string[] contains)
        {
            if(document == null) return null;
            try {
                if(contains == null || contains.Length == 0) return null;
                if(contains.Length == 1) {
                    if(tag.Length == 0) {
                        foreach(HtmlElement el in document.All)
                            if(el.OuterHtml.Contains(contains[0])) return el;
                    } else {
                        foreach(HtmlElement el in document.All)
                            if(el.TagName == tag && el.OuterHtml.Contains(contains[0])) return el;
                    }
                } else {
                    foreach(string part in contains) if(part.Length == 0) return null;
                    if(tag.Length == 0) {
                        foreach(HtmlElement el in document.All) {
                            bool hasAll = true;
                            foreach(string part in contains)
                                if(el.OuterHtml.Contains(part) == false) { hasAll = false; break; }
                            if(hasAll) return el;
                        }
                    } else {
                        foreach(HtmlElement el in document.All) {
                            bool hasAll = true;
                            foreach(string part in contains)
                                if(el.TagName != tag || el.OuterHtml.Contains(part) == false) { hasAll = false; break; }
                            if(hasAll) return el;
                        }
                    }
                }
                if(document.Window.Frames.Count > 0)
                    foreach(HtmlWindow window in document.Window.Frames) {
                        HtmlElement el = GetElementThatContains(window.Document, tag, contains);
                        if(el != null) return el;
                    }
            } catch { }
            return null;
        }
        public static HtmlElement GetElementThatContains(WebBrowser webBrowser, string tag, string[] contains)
        {
            return GetElementThatContains(webBrowser != null ? webBrowser.Document : null, tag, contains);
        }


        /// <summary>
        /// -1 not selected, -2 error
        /// </summary>
        public static int GetSelectedIndex(HtmlElement element)
        {
            if(element == null) return -2;
            try {
                if(element.TagName.ToUpper() == "SELECT") {
                    int index = -2;
                    if(Int32.TryParse(element.GetAttribute("selectedIndex"), out index)) return index;
                }
            } catch { }
            return -2;            
        }

        public static bool SetSelectedIndex(HtmlElement element, int selectIndex)
        {
            if(element == null) return false;
            try {
                if(element.TagName.ToUpper() == "SELECT") {
                    element.SetAttribute("selectedIndex", selectIndex.ToString());
                    return true;
                }
            } catch { }
            return false;
        }

        public static bool SetText(HtmlElement element, string text)
        {
            if(element == null) return false;
            try {
                if(element.TagName.ToUpper() == "INPUT") {
                    element.SetAttribute("value", text);
                    return true;
                }
            } catch { }
            return false;
        }

        public static bool ButtonClick(HtmlElement element)
        {
            if(element == null) return false;
            try {
                if(element.TagName.ToUpper() == "INPUT") {
                    element.InvokeMember("click");
                    return true;
                }
            } catch { }
            return false;
        }


        /*
        public static Boolean HrefClick(HtmlElement element)
        {
            if(element == null) return false;
            try {
                HTMLHRElement domel = (HTMLHRElement)element.DomElement;
                domel.click();
                return true;
            } catch(Exception) { }
            return false;
        }
        */

        public static void ShowElements(HtmlDocument doc)
        {
            try {
                FormElements form = new FormElements();
                form.UpdateList(doc);
                form.ShowDialog();
                form.Dispose();
            } catch { }
        }
        public static void ShowElements(WebBrowser wb)
        {
            if(wb != null && wb.Document != null) ShowElements(wb.Document);
        }

        #region panel PanelElements for ShowElements

        private class PanelElements: Panel
        {

            private System.Windows.Forms.ListBox log;
            private System.Windows.Forms.TextBox text2;
            private System.Windows.Forms.TextBox text1;
            private System.Windows.Forms.ListView list;
            private System.Windows.Forms.ColumnHeader colFrame;
            private System.Windows.Forms.ColumnHeader colTag;
            private System.Windows.Forms.ColumnHeader colName;
            private System.Windows.Forms.ColumnHeader colID;
            private System.Windows.Forms.ColumnHeader colValue;
            private System.Windows.Forms.ColumnHeader colHTML;

            public PanelElements()
            {
                this.log = new System.Windows.Forms.ListBox();
                this.text2 = new System.Windows.Forms.TextBox();
                this.text1 = new System.Windows.Forms.TextBox();
                this.list = new System.Windows.Forms.ListView();
                this.colFrame = new System.Windows.Forms.ColumnHeader();
                this.colTag = new System.Windows.Forms.ColumnHeader();
                this.colName = new System.Windows.Forms.ColumnHeader();
                this.colID = new System.Windows.Forms.ColumnHeader();
                this.colValue = new System.Windows.Forms.ColumnHeader();
                this.colHTML = new System.Windows.Forms.ColumnHeader();
                this.SuspendLayout();
                // 
                // log
                // 
                this.log.FormattingEnabled = true;
                this.log.Location = new System.Drawing.Point(327, 31);
                this.log.Name = "log";
                this.log.Size = new System.Drawing.Size(142, 264);
                this.log.TabIndex = 8;
                this.log.Visible = false;
                // 
                // text2
                // 
                this.text2.Dock = System.Windows.Forms.DockStyle.Bottom;
                this.text2.Location = new System.Drawing.Point(0, 432);
                this.text2.Multiline = true;
                this.text2.Name = "text2";
                this.text2.ScrollBars = System.Windows.Forms.ScrollBars.Both;
                this.text2.Size = new System.Drawing.Size(496, 114);
                this.text2.TabIndex = 14;
                // 
                // text1
                // 
                this.text1.Dock = System.Windows.Forms.DockStyle.Bottom;
                this.text1.Location = new System.Drawing.Point(0, 318);
                this.text1.Multiline = true;
                this.text1.Name = "text1";
                this.text1.ScrollBars = System.Windows.Forms.ScrollBars.Both;
                this.text1.Size = new System.Drawing.Size(496, 114);
                this.text1.TabIndex = 15;
                // 
                // list
                // 
                this.list.AutoArrange = false;
                this.list.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
                    this.colFrame,
                    this.colTag,
                    this.colName,
                    this.colID,
                    this.colValue,
                    this.colHTML});
                this.list.Dock = System.Windows.Forms.DockStyle.Fill;
                this.list.FullRowSelect = true;
                this.list.GridLines = true;
                this.list.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
                this.list.Location = new System.Drawing.Point(0, 0);
                this.list.Name = "list";
                this.list.ShowGroups = false;
                this.list.Size = new System.Drawing.Size(496, 318);
                this.list.TabIndex = 16;
                this.list.UseCompatibleStateImageBehavior = false;
                this.list.View = System.Windows.Forms.View.Details;
                this.list.SelectedIndexChanged += new System.EventHandler(this.list_SelectedIndexChanged);
                // 
                // colFrame
                // 
                this.colFrame.Text = "Frame";
                // 
                // colTag
                // 
                this.colTag.Text = "Tag";
                this.colTag.Width = 67;
                // 
                // colName
                // 
                this.colName.Text = "Name";
                this.colName.Width = 63;
                // 
                // colID
                // 
                this.colID.Text = "ID";
                // 
                // colValue
                // 
                this.colValue.Text = "Value";
                // 
                // colHTML
                // 
                this.colHTML.Text = "HTML";
                this.colHTML.Width = 166;
                // 
                // PanelElements
                // 
                this.Controls.Add(this.list);
                this.Controls.Add(this.text1);
                this.Controls.Add(this.text2);
                this.Controls.Add(this.log);
                this.Name = "PanelElements";
                this.ResumeLayout(false);
                this.PerformLayout(); 
            }

            private void list_SelectedIndexChanged(object sender, EventArgs e)
            {
                HtmlElement el = null;
                try { el = ((ListViewItemTag)list.SelectedItems[0].Tag).el; } catch(Exception) { }
                String text = "";
                // show InnerHtml
                try { if(el != null) text = el.OuterHtml; } catch(Exception) { }
                if(this.text1.Text != text) this.text1.Text = text;
                // show attributes
                text = "";
                try {
                    if(el != null) {
                        String[] attribs = el.OuterHtml.Split(' ');
                        foreach(String attrib in attribs)
                            if(attrib.Contains("=")) {
                                String s = attrib.Substring(0, attrib.IndexOf('='));
                                text += s + " = " + el.GetAttribute(s) + "\r\n";
                            }
                        text += "styles: " + el.Style;
                    }
                } catch(Exception) { }
                if(this.text2.Text != text) this.text2.Text = text;
            }

            private int sub = 0;
            private void setLog(int index, String text)
            {
                while(log.Items.Count < (index + 1)) log.Items.Add("");
                log.Items[index] = text;
            }

            public void UpdateList(HtmlDocument doc)
            {
                if(doc == null) { list.Items.Clear(); return; }
                SetProcessed(false);
                if(log.Visible) {
                    sub = 0;
                    log.Items.Clear();
                }
                UpdateWindow(doc.Window);
                RemoveNotProcessed();
                Application.DoEvents();
            }

            public void UpdateWindow(HtmlWindow window)
            {
                if(log.Visible) {
                    String S = "";
                    try { S += "name: '" + window.Name + "'"; } catch(Exception) { S += "name: err"; }
                    try { S += " doc: " + (window.Document != null ? "ok" : "null"); } catch(Exception) { S += " doc: err"; }
                    try { S += " el: " + (window.WindowFrameElement != null ? "ok" : "null"); } catch(Exception) { S += " el: err"; }
                    try { S += " size: " + window.Size.ToString(); } catch(Exception) { S += " size: err"; }
                    S += " frames: " + window.Frames.Count;
                    setLog(sub++, S);
                }

                try {
                    HtmlDocument doc = window.Document;
                    if(doc != null) {
                        String frameName = window.Name;
                        foreach(HtmlElement el in doc.All) {
                            String tag = (el.TagName == null ? "" : el.TagName);
                            String name = (el.Name == null ? "" : el.Name);
                            String id = (el.Id == null ? "" : el.Id);
                            String value = el.GetAttribute("Value");
                            String html = (el.OuterHtml == null ? "" : el.OuterHtml);

                            ListViewItem item = GetListViewItem(el);
                            if(item == null) {
                                item = list.Items.Add(frameName);
                                item.SubItems.Add(tag);
                                item.SubItems.Add(name);
                                item.SubItems.Add(id);
                                item.SubItems.Add(value);
                                item.SubItems.Add(html);
                            } else {
                                if(item.SubItems[0].Text != frameName) item.SubItems[0].Text = frameName;
                                if(item.SubItems[1].Text != tag) item.SubItems[1].Text = tag;
                                if(item.SubItems[2].Text != name) item.SubItems[2].Text = name;
                                if(item.SubItems[3].Text != id) item.SubItems[3].Text = id;
                                if(item.SubItems[4].Text != value) item.SubItems[4].Text = value;
                                if(item.SubItems[5].Text != html) item.SubItems[5].Text = html;
                            }
                            item.Tag = new ListViewItemTag(el, true);
                            Application.DoEvents();
                        }
                    }
                } catch(Exception) { }
                try {
                    if(window.Frames.Count > 0)
                        foreach(HtmlWindow frame in window.Frames) {
                            UpdateWindow(frame);
                        }
                } catch(Exception) { }
            }

            private void SetProcessed(Boolean processed)
            {
                foreach(ListViewItem item in list.Items)
                    ((ListViewItemTag)item.Tag).processed = processed;
            }

            private ListViewItem GetListViewItem(HtmlElement el)
            {
                foreach(ListViewItem item in list.Items)
                    if(((ListViewItemTag)item.Tag).el.Equals(el)) return item;
                return null;
            }

            private void RemoveNotProcessed()
            {
                foreach(ListViewItem item in list.Items)
                    if(((ListViewItemTag)item.Tag).processed == false)
                        list.Items.Remove(item);
            }


            private class ListViewItemTag
            {
                public HtmlElement el = null;
                public Boolean processed = false;
                public ListViewItemTag(HtmlElement el, Boolean processed)
                {
                    this.el = el;
                    this.processed = processed;
                }
            }

        }
        #endregion

        #region class FormElements for method ShowElements

        private class FormElements: Form
        {

            #region Windows Form Designer generated code

            /// <summary>
            /// Required method for Designer support - do not modify
            /// the contents of this method with the code editor.
            /// </summary>
            private void InitializeComponent()
            {
                this.panelElements = new PanelElements();
                this.SuspendLayout();
                // 
                // log
                // 
                this.panelElements.Dock = DockStyle.Fill;
                this.panelElements.Location = new System.Drawing.Point(327, 31);
                this.panelElements.Name = "panelElements";
                this.panelElements.Size = new System.Drawing.Size(142, 264);
                this.panelElements.TabIndex = 0;
                // 
                // FormElements
                // 
                this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
                this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
                this.ClientSize = new System.Drawing.Size(496, 546);
                this.Controls.Add(this.panelElements);
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow;
                this.Name = "FormElements";
                this.Text = "Elements";
                this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.FormElements_FormClosing);
                this.ResumeLayout(false);
                this.PerformLayout();

            }

            #endregion

            private PanelElements panelElements;

            public FormElements()
            {
                InitializeComponent();
                this.TopMost = true;
            }
            public void UpdateList(HtmlDocument doc)
            {
                panelElements.UpdateList(doc);
            }

            public void UpdateWindow(HtmlWindow window)
            {
                panelElements.UpdateWindow(window);
            }


            private void FormElements_FormClosing(object sender, FormClosingEventArgs e)
            {
                e.Cancel = true;
                this.Hide();
            }

        }

        #endregion

        public static void ShowBrowser(WebBrowser wb, bool showElements)
        {
            try {
                FormBrowser form = new FormBrowser(wb, showElements);
                form.ShowDialog();
                form.Dispose();
            } catch { }
        }

        #region FormBrowser for method ShowBrowser

        private class FormBrowser: Form
        {
            private SplitContainer splitContainer;
            private WebBrowser webBrowser;
            private PanelElements panelElements;

            public FormBrowser(WebBrowser webBrowser, bool showElements)
            {
                this.splitContainer = new SplitContainer();
                this.webBrowser = webBrowser;
                this.SuspendLayout();
                //
                // splitContainer
                //
                this.splitContainer.Dock = DockStyle.Fill;
                this.splitContainer.Panel1.Controls.Add(webBrowser);
                if(showElements) {
                    this.panelElements = new PanelElements();
                    this.panelElements.Dock = DockStyle.Fill;
                    this.splitContainer.Panel2.Controls.Add(this.panelElements);
                    this.splitContainer.SplitterDistance = 580;
                } else {
                    this.splitContainer.Panel2Collapsed = true;
                }
                // 
                // webBrowser
                // 
                this.webBrowser.Dock = System.Windows.Forms.DockStyle.Fill;
                this.webBrowser.Location = new System.Drawing.Point(0, 0);
                this.webBrowser.MinimumSize = new System.Drawing.Size(20, 20);
                this.webBrowser.Name = "webBrowser";
                this.webBrowser.Size = new System.Drawing.Size(644, 460);
                this.webBrowser.TabIndex = 0;
                // 
                // FormTest
                // 
                this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
                this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
                this.ClientSize = new System.Drawing.Size(1024, 768);
                this.Controls.Add(this.splitContainer);
                this.Name = "FormBrowser";
                this.Text = "Browser";
                this.ResumeLayout(false);

                this.FormClosing += new FormClosingEventHandler(FormBrowser_FormClosing);
                if(showElements && panelElements != null && webBrowser != null) this.panelElements.UpdateList(webBrowser.Document);
            }

            void FormBrowser_FormClosing(object sender, FormClosingEventArgs e)
            {
                this.splitContainer.Panel1.Controls.Remove(webBrowser);
            }

        }
        #endregion


    }

    #region ExtendedWebBrowser

    public class ExtendedWebBrowser: System.Windows.Forms.WebBrowser
    {
        System.Windows.Forms.AxHost.ConnectionPointCookie cookie;
        WebBrowserExtendedEvents events;

        //This method will be called to give you a chance to create your own event sink
        protected override void CreateSink()
        {
            //MAKE SURE TO CALL THE BASE or the normal events won't fire
            base.CreateSink();
            events = new WebBrowserExtendedEvents(this);
            cookie = new System.Windows.Forms.AxHost.ConnectionPointCookie(this.ActiveXInstance, events, typeof(DWebBrowserEvents2));
        }

        protected override void DetachSink()
        {
            if(null != cookie) {
                cookie.Disconnect();
                cookie = null;
            }
            base.DetachSink();
        }

        //This new event will fire when the page is navigating
        public event EventHandler<WebBrowserExtendedNavigatingEventArgs> BeforeNavigate;
        public event EventHandler<WebBrowserExtendedNavigatingEventArgs> BeforeNewWindow;

        protected void OnBeforeNewWindow(string url, out bool cancel)
        {
            EventHandler<WebBrowserExtendedNavigatingEventArgs> h = BeforeNewWindow;
            WebBrowserExtendedNavigatingEventArgs args = new WebBrowserExtendedNavigatingEventArgs(url, null);
            if(null != h) {
                h(this, args);
            }
            cancel = args.Cancel;
        }

        protected void OnBeforeNavigate(string url, string frame, out bool cancel)
        {
            EventHandler<WebBrowserExtendedNavigatingEventArgs> h = BeforeNavigate;
            WebBrowserExtendedNavigatingEventArgs args = new WebBrowserExtendedNavigatingEventArgs(url, frame);
            if(null != h) {
                h(this, args);
            }
            //Pass the cancellation chosen back out to the events
            cancel = args.Cancel;
        }
        //This class will capture events from the WebBrowser
        class WebBrowserExtendedEvents: System.Runtime.InteropServices.StandardOleMarshalObject, DWebBrowserEvents2
        {
            ExtendedWebBrowser _Browser;
            public WebBrowserExtendedEvents(ExtendedWebBrowser browser) { _Browser = browser; }

            //Implement whichever events you wish
            public void BeforeNavigate2(object pDisp, ref object URL, ref object flags, ref object targetFrameName, ref object postData, ref object headers, ref bool cancel)
            {
                _Browser.OnBeforeNavigate((string)URL, (string)targetFrameName, out cancel);
            }

            public void NewWindow3(object pDisp, ref bool cancel, ref object flags, ref object URLContext, ref object URL)
            {
                _Browser.OnBeforeNewWindow((string)URL, out cancel);
            }

        }
        [System.Runtime.InteropServices.ComImport(), System.Runtime.InteropServices.Guid("34A715A0-6587-11D0-924A-0020AFC7AC4D"),
        System.Runtime.InteropServices.InterfaceTypeAttribute(System.Runtime.InteropServices.ComInterfaceType.InterfaceIsIDispatch),
        System.Runtime.InteropServices.TypeLibType(System.Runtime.InteropServices.TypeLibTypeFlags.FHidden)]
        public interface DWebBrowserEvents2
        {

            [System.Runtime.InteropServices.DispId(250)]
            void BeforeNavigate2(
                [System.Runtime.InteropServices.In,
                System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.IDispatch)] object pDisp,
                [System.Runtime.InteropServices.In] ref object URL,
                [System.Runtime.InteropServices.In] ref object flags,
                [System.Runtime.InteropServices.In] ref object targetFrameName, [System.Runtime.InteropServices.In] ref object postData,
                [System.Runtime.InteropServices.In] ref object headers,
                [System.Runtime.InteropServices.In,
                System.Runtime.InteropServices.Out] ref bool cancel);
            [System.Runtime.InteropServices.DispId(273)]
            void NewWindow3(
                [System.Runtime.InteropServices.In,
                System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.IDispatch)] object pDisp,
                [System.Runtime.InteropServices.In, System.Runtime.InteropServices.Out] ref bool cancel,
                [System.Runtime.InteropServices.In] ref object flags,
                [System.Runtime.InteropServices.In] ref object URLContext,
                [System.Runtime.InteropServices.In] ref object URL);

        }
    }

    public class WebBrowserExtendedNavigatingEventArgs: System.ComponentModel.CancelEventArgs
    {
        private string _Url;
        public string Url
        {
            get { return _Url; }
        }

        private string _Frame;
        public string Frame
        {
            get { return _Frame; }
        }

        public WebBrowserExtendedNavigatingEventArgs(string url, string frame)
            : base()
        {
            _Url = url;
            _Frame = frame;
        }
    }


    #endregion
}
